import os
import re

from piper.config.sanitizers import asserter


_BASE_CHARS = r"[A-Za-z0-9\-\._]+"
_PACKAGE = re.compile(r"^"+_BASE_CHARS+r"$")
_VERSION = re.compile(r"^"+_BASE_CHARS+r"$")  # To be changed or not?

_MODIFIERS = r"(==|<|<=|>|>=|!=|===|~=)"
_VERSIONS_M = r"[A-Za-z0-9\-\._\*]+"
_VERSIONED_PACKAGE = re.compile(r"^"+_BASE_CHARS+r"("+_MODIFIERS+_VERSIONS_M+", *)*("+_MODIFIERS+_VERSIONS_M+")? *$")


def sanitize_package(package: str) -> str:
    return asserter.regex(_PACKAGE, package)


def is_just_package(package: str) -> bool:
    return bool(_PACKAGE.match(package))


def sanitize_version(version: str, accept_path=False) -> str:
    if accept_path and os.path.isdir(version):
        return version
    else:
        return asserter.regex(_VERSION, version)


def sanitize_versioned_package(spec: str) -> str:
    return asserter.regex(_VERSIONED_PACKAGE, spec)



import argparse
import importlib
import inspect
import os
import re
from pathlib import Path

import yaml
from typing import Dict, Any
from pipertask.task import PiperTask


def _read_config(context: str, name: str) -> (Dict[str, Any], str):
    # Read the pipe file
    with open(os.path.join(context, "pipe.yml")) as p:
        pipe = yaml.load(p, Loader=yaml.SafeLoader) or {}

    # Get all the tasks with matching name
    matching = [
        task
        for step in pipe.get("tasks", {}).values()
        for tasks in step.values()
        for task in tasks
        if task.get("name") == name
    ]
    if len(matching) != 1:
        raise ValueError(f"There should be a single task named {name}")

    # Get package name
    package = re.search(r"^([a-zA-Z0-9\-\._]+)", matching[0]["pipertask"]).group(1)

    # Return the config
    return matching[0].get("config", {}), package


def perform(context, name):

    # Get configs
    config, package = _read_config(context, name)

    # Import the task class
    module = f"{package}.task"
    all_tasks = inspect.getmembers(
        importlib.import_module(module),
        lambda member: inspect.isclass(member) and member.__module__ == module and issubclass(member, PiperTask)
    )[0]
    task = all_tasks[1]

    # Perform the task
    print(f"   TASK {name}   ".center(60, '*'))
    task().perform(Path(context), config)
    print("*"*60)


def main():

    # Parse
    parser = argparse.ArgumentParser()
    parser.add_argument("--context")
    parser.add_argument("--task")
    args = parser.parse_args()

    perform(args.context, args.task)


if __name__ == '__main__':

    main()



from typing import List, Union

from piper.config.reader import read_all_pipes
from piper.config.sanitizers.pip_sanitizer import sanitize_package, sanitize_versioned_package, is_just_package
from piper.tools.executable import Executable


class Pip(Executable):

    def __init__(self, pre_command: str = None):
        super().__init__("pip", pre_command=pre_command)

    def install(self, requirements: Union[List[str], str], allow_pipes_from=None):

        if isinstance(requirements, str):
            requirements = [requirements]

        # Install real requirements
        packages = [
            f'{sanitize_versioned_package(req)}'
            for req in requirements
            if not is_just_package(req)
        ]
        if packages:
            super().run(f"install {' '.join(packages)}")

        # Install pipes
        pipes = [
            f'{sanitize_package(req)}'
            for req in requirements
            if is_just_package(req)
        ]
        if pipes:
            if not allow_pipes_from:
                raise ValueError(f"{pipes} without versions (or pipes) are not allowed here")
            else:
                all_pipes = read_all_pipes(allow_pipes_from)
                super().run(f"install -e {' '.join([p.setup_py_folder for p in all_pipes.values() if p.name in pipes])}")



import logging
import os

import pkg_resources
from piper import RESOURCES_LOCATION
from piper.config.pipe import Pipe
from piper.config.sanitizers import pip_sanitizer
from piper.tools.venv import Virtualenv

_PERFORM_TASK = os.path.join(RESOURCES_LOCATION, "tasks", "perform.py")


logger = logging.getLogger(__name__)


def perform_tasks(phase: str, step: str, pipe: Pipe):

    # Get tasks
    tasks = pipe.tasks.get(phase, {}).get(step, {})
    if tasks:

        # Create the env for tasks # TODO could be different venvs to avoid conflicts?
        logger.info("*"*60)
        logger.info(f"   TASKS {phase}: {step}   ".center(60, "*"))
        logger.info("*" * 60)
        venv = Virtualenv(Pipe(location=pipe.tasks_folder, yml={}))
        venv.create()

        # Install PyYAML for the performer
        version = pip_sanitizer.sanitize_version(pkg_resources.get_distribution("PyYAML").version)
        venv.pip.install(f"PyYAML=={version}")

        # Install the pipertasks
        venv.pip.install([t.pipertask for t in tasks], allow_pipes_from=pipe.root_context)

        # Perform each task in order
        for task in tasks:
            venv.python.run(f"{_PERFORM_TASK} --context {pipe.location} --task {task.name}")
        logger.info(f"   END TASKS {phase}: {step}   ".center(60, "*"))
        logger.info("*"*60)



from piper.config.sanitizers.name_sanitizer import sanitize_name
from piper.config.sanitizers.pip_sanitizer import sanitize_versioned_package


class Task:

    def __init__(self, yml):
        self.pipertask = sanitize_versioned_package(yml.pop("pipertask"))
        self.name = sanitize_name(yml.pop("name"))
        self.config = yml.pop("config", {})

        # If there are still configs, they are unknown. Raise an error
        if yml:
            raise KeyError(f"Unknown configuration in pipe file, task {self.name}: {yml}")
